function [draw,finalaccept,postdraw,priorcount,eucount] = MHalgorithm(obsdata,simlen,mode,SIG,nvar,nforerrors,nexog,obs,bnames)

tic
disp('Beginning MH algorithm')

% General Information for the Kalman Filter
nobs = size(obsdata,1);
T = size(obsdata,2);
initX = zeros(nvar,1);

% Initializations for draws
gamm100draw = zeros(simlen,1);   
kappadraw = zeros(simlen,1);      
omegawdraw = zeros(simlen,1);   
omegapdraw = zeros(simlen,1);  
phipidraw = zeros(simlen,1);    
phiydraw = zeros(simlen,1); 
thetadraw = zeros(simlen,1);
psidraw = zeros(simlen,1); 
sdraw = zeros(simlen,1);  
chiwdraw = zeros(simlen,1);     
rhoadraw = zeros(simlen,1);
rhobdraw = zeros(simlen,1);
rhordraw = zeros(simlen,1);
rhoidraw = zeros(simlen,1);
rholabdraw = zeros(simlen,1);
rhowdraw = zeros(simlen,1);
rhopdraw = zeros(simlen,1);
rhogdraw = zeros(simlen,1);
rhotdraw = zeros(simlen,1);
rhotcdraw = zeros(simlen,1);
rhozdraw = zeros(simlen,1);
gammgdraw = zeros(simlen,1);
gammtdraw = zeros(simlen,1);
gammtcdraw = zeros(simlen,1);
gammwgdraw = zeros(simlen,1);
sigadraw = zeros(simlen,1);
sigbdraw = zeros(simlen,1);
sigmdraw = zeros(simlen,1);
sigidraw = zeros(simlen,1);
siglabdraw = zeros(simlen,1);
sigwdraw = zeros(simlen,1);
sigpdraw = zeros(simlen,1);
siggdraw = zeros(simlen,1);
sigtdraw = zeros(simlen,1);
sigtcdraw = zeros(simlen,1);
sigzdraw = zeros(simlen,1);
sigwgdraw = zeros(simlen,1);
rhowgdraw = zeros(simlen,1);
kappawpdraw = zeros(simlen,1);
mundraw = zeros(simlen,1);
thetaepdraw = zeros(simlen,1);
thetaewdraw = zeros(simlen,1);
thetaLdraw = zeros(simlen,1);

postdraw = zeros(simlen,1);     % posterior likelihood at each draw
monitor = 0;                    %
priorcount = 0;                 % number of times draw is outside prior distribution
eucount = 0;                    %

    save mhresults.mat gamm100draw kappadraw omegawdraw omegapdraw...
        psidraw sdraw chiwdraw phipidraw phiydraw thetadraw ...
        gammgdraw gammtdraw gammtcdraw gammwgdraw ...
        rhoadraw rhobdraw rhordraw rhoidraw rholabdraw rhowdraw rhopdraw rhogdraw...
        rhotdraw rhotcdraw rhozdraw rhowgdraw sigadraw sigbdraw...
        sigmdraw sigidraw siglabdraw sigwdraw sigpdraw siggdraw sigtdraw sigtcdraw ...
        sigzdraw sigwgdraw kappawpdraw mundraw thetaepdraw thetaewdraw thetaLdraw...
        postdraw monitor priorcount eucount
 

% Info for MH algorithm
acceptcount = 0;                % counts how many draws accepted
acceptblock = 0;
canddraw = zeros(size(mode));
rep = 100;                      % how many iterations to check acceptance rate

% TUNING PARAMETERS
c0 = 1;                         % initial value of tuning parameter 
c = 0.31;                        % MH algorithm tuning parameter 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%       Initial Values         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lastdraw = mvnrnd(mode,(c0^2).*SIG)';   % Initial Draw 

priorlast = priors(lastdraw');          % Prior at initial value
while priorlast == -Inf
    lastdraw = mvnrnd(mode,(c0^2).*SIG)';
    priorlast = priors(lastdraw');
end
% Get Initial Likelihood Value 
[g0,g1,CC,Psi,Pie,H,C,E,ssvalues,ssnames,bad_steady_state] = model(lastdraw,bnames,obs,glevelI);
while bad_steady_state == 1
    lastdraw = mvnrnd(mode,(c0^2).*SIG)';   
    priorlast = priors(lastdraw');
    while priorlast == -Inf
        lastdraw = mvnrnd(mode,(c0^2).*SIG)';
        priorlast = priors(lastdraw');
    end
    [g0,g1,CC,Psi,Pie,H,C,E,ssvalues,ssnames,bad_steady_state] = model(lastdraw,bnames,obs,glevelI);
end
    [G,C0,M,fmat,fwt,ywt,gev,eu] = gensys(g0,g1,CC,Psi,Pie);
while eu(1,1) ~= 1 || eu(2,1) ~= 1      % must have unique equilibrium
    lastdraw = mvnrnd(mode,(c0^2).*SIG)';   
    priorlast = priors(lastdraw');
    while priorlast == -Inf
        lastdraw = mvnrnd(mode,(c0^2).*SIG)';
        priorlast = priors(lastdraw');
    end
    [g0,g1,CC,Psi,Pie,H,C,E,ssvalues,ssnames,bad_steady_state] = model(lastdraw,bnames,obs,glevelI);
    while bad_steady_state == 1
    lastdraw = mvnrnd(mode,(c0^2).*SIG)';   
    priorlast = priors(lastdraw');
    while priorlast == -Inf
        lastdraw = mvnrnd(mode,(c0^2).*SIG)';
        priorlast = priors(lastdraw');
    end
    [g0,g1,CC,Psi,Pie,H,C,E,ssvalues,ssnames,bad_steady_state] = model(lastdraw,bnames,obs,glevelI);
    end
    [G,C0,M,fmat,fwt,ywt,gev,eu] = gensys(g0,g1,CC,Psi,Pie);
end
M=M(:,nvar-nexog+1:end);
Q = M*E*E'*M';
initP = lyapunov_symm(G,Q,1); % lyapunov_symm is a dynare code that solves a Lyapunov equation symmetrically
lastlike = kalman(obsdata,G,Q,initP,initX,T,nobs,obs,C);  
postlast = priorlast + lastlike;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Run Algorithm        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('Beginning Loop')
for i = 1:simlen
    canddraw = lastdraw + mvnrnd(zeros(size(mode)),(c^2).*SIG)';

    priorcand = priors(canddraw');
    if priorcand == -Inf
        priorcount = priorcount + 1;
        postcand = -100000;
    else
    [g0,g1,CC,Psi,Pie,H,C,E,ssvalues,ssnames,bad_steady_state] = model(canddraw,bnames,obs,glevelI);
    if bad_steady_state == 1
        eucount = eucount + 1;
        postcand = -100000;
    else
    [G,C0,M,fmat,fwt,ywt,gev,eu] = gensys(g0,g1,CC,Psi,Pie);
    M=M(:,nvar-nexog+1:end);
    if eu(1,1) ~= 1 || eu(2,1) ~= 1
        eucount = eucount + 1;
        % If no determinate equilibrium, accept last draw instead of
        % candidate.
        postcand = -100000; 
    else
        % Get Candidate Likelihood
        Q = M*E*E'*M';
        initP = lyapunov_symm(G,Q,1); % lyapunov_symm is a dynare code that solves a Lyapunov equation symmetrically
        candlike = kalman(obsdata,G,Q,initP,initX,T,nobs,obs,C);
       
        postcand = priorcand + candlike;
    end
    end
    end
    if min(exp(postcand - postlast),1) > rand(1,1) 
        % Accept candidate draw in this case.  Update last values.
        lastdraw = canddraw;
        postlast = postcand;
        acceptcount = acceptcount + 1;
        acceptblock = acceptblock + 1;
    end    
  
    
    %% gamm100
    param_index = 1;
    gamm100draw(i,1) = lastdraw(param_index,1);     
    %% inverse Frisch
    param_index = param_index + 1;
    kappadraw(i,1) = lastdraw(param_index,1);      
    %% omegaw
    param_index = param_index + 1;
    omegawdraw(i,1) = lastdraw(param_index,1);     
    %% omegap
    param_index = param_index + 1;
    omegapdraw(i,1) = lastdraw(param_index,1);        
    %% phipi
    param_index = param_index + 1;
    phipidraw(i,1) = lastdraw(param_index,1);    
    %% phiy
    param_index = param_index + 1;
    phiydraw(i,1) = lastdraw(param_index,1);  
    %% theta
    param_index = param_index + 1;
    thetadraw(i,1) = lastdraw(param_index,1);  
    %% psi
    param_index = param_index + 1;
    psidraw(i,1) = lastdraw(param_index,1);    
    %% s
    param_index = param_index + 1;
    sdraw(i,1) = lastdraw(param_index,1);    
    %% chiw
    param_index = param_index + 1;
    chiwdraw(i,1) = lastdraw(param_index,1);   
    %% rhoa
    param_index = param_index + 1;
    rhoadraw(i,1) = lastdraw(param_index,1); 
    %% rhob
    param_index = param_index + 1;
    rhobdraw(i,1) = lastdraw(param_index,1); 
    %% rhor
    param_index = param_index + 1;
    rhordraw(i,1) = lastdraw(param_index,1);
    %% rhoi
    param_index = param_index + 1;
    rhoidraw(i,1) = lastdraw(param_index,1); 
    %% rholab
    param_index = param_index + 1;
    rholabdraw(i,1) = lastdraw(param_index,1);
    %% rhow
    param_index = param_index + 1;
    rhowdraw(i,1) = lastdraw(param_index,1);
    %% rhop
    param_index = param_index + 1;
    rhopdraw(i,1) = lastdraw(param_index,1); 
    %% rhog
    param_index = param_index + 1;
    rhogdraw(i,1) = lastdraw(param_index,1); 
    %% rhot
    param_index = param_index + 1;
    rhotdraw(i,1) = lastdraw(param_index,1); 
    %% rhotc
    param_index = param_index + 1;
    rhotcdraw(i,1) = lastdraw(param_index,1); 
    %% rhoz
    param_index = param_index + 1;
    rhozdraw(i,1) = lastdraw(param_index,1); 
    %% gammg
    param_index = param_index + 1;
    gammgdraw(i,1) = lastdraw(param_index,1); 
    %% gammt
    param_index = param_index + 1;
    gammtdraw(i,1) = lastdraw(param_index,1); 
    %% gammtc
    param_index = param_index + 1;
    gammtcdraw(i,1) = lastdraw(param_index,1); 
    %% gammwg
    param_index = param_index + 1;
    gammwgdraw(i,1) = lastdraw(param_index,1); 
    %% siga
    param_index = param_index + 1;
    sigadraw(i,1) = lastdraw(param_index,1); 
    %% sigb
    param_index = param_index + 1;
    sigbdraw(i,1) = lastdraw(param_index,1); 
    %% sigm
    param_index = param_index + 1;
    sigmdraw(i,1) = lastdraw(param_index,1); 
    %% sigi
    param_index = param_index + 1;
    sigidraw(i,1) = lastdraw(param_index,1); 
    %% siglab
    param_index = param_index + 1;
    siglabdraw(i,1) = lastdraw(param_index,1); 
    %% sigw
    param_index = param_index + 1;
    sigwdraw(i,1) = lastdraw(param_index,1); 
    %% sigp
    param_index = param_index + 1;
    sigpdraw(i,1) = lastdraw(param_index,1); 
    %% sigg
    param_index = param_index + 1;
    siggdraw(i,1) = lastdraw(param_index,1); 
    %% sigt
    param_index = param_index + 1;
    sigtdraw(i,1) = lastdraw(param_index,1); 
    %% sigtc
    param_index = param_index + 1;
    sigtcdraw(i,1) = lastdraw(param_index,1); 
    %% sigz
    param_index = param_index + 1;
    sigzdraw(i,1) = lastdraw(param_index,1); 
    %% sigwg
    param_index = param_index + 1;
    sigwgdraw(i,1) = lastdraw(param_index,1); 
    %% rhowg
    param_index = param_index + 1;
    rhowgdraw(i,1) = lastdraw(param_index,1); 
    %% kappawp
    param_index = param_index + 1;
    kappawpdraw(i,1) = lastdraw(param_index,1);
    %% mun
    param_index = param_index + 1;
    mundraw(i,1) = lastdraw(param_index,1);
    %% thetaep
    param_index = param_index + 1;
    thetaepdraw(i,1) = lastdraw(param_index,1);
    %% thetaew
    param_index = param_index + 1;
    thetaewdraw(i,1) = lastdraw(param_index,1);
    %% thetaL
    param_index = param_index + 1;
    thetaLdraw(i,1) = lastdraw(param_index,1);
    
    postdraw(i,1) = postlast;  
    
    if i/rep == round(i/rep)
        monitor = acceptcount/i;  % monitors acceptance rate
        disp(monitor)
    end
    if i/10000 == round(i/10000)
        % save as you go in case of crash
    save mhresults.mat gamm100draw kappadraw omegawdraw omegapdraw...
        psidraw sdraw chiwdraw phipidraw phiydraw thetadraw ...
        gammgdraw gammtdraw gammtcdraw gammwgdraw ...
        rhoadraw rhobdraw rhordraw rhoidraw rholabdraw rhowdraw rhopdraw rhogdraw...
        rhotdraw rhotcdraw rhozdraw rhowgdraw sigadraw sigbdraw...
        sigmdraw sigidraw siglabdraw sigwdraw sigpdraw siggdraw sigtdraw sigtcdraw ...
        sigzdraw sigwgdraw kappawpdraw mundraw thetaepdraw thetaewdraw thetaLdraw...
        postdraw monitor priorcount eucount -append
    end
    if (i/rep) == round(i/rep)
        disp(acceptblock/rep)
        disp(i/rep)
        acceptblock = 0;
    end
end

draw = [gamm100draw kappadraw omegawdraw omegapdraw phipidraw phiydraw ...
        thetadraw psidraw sdraw chiwdraw ...
        rhoadraw rhobdraw rhordraw rhoidraw rholabdraw rhowdraw rhopdraw rhogdraw...
        rhotdraw rhotcdraw rhozdraw gammgdraw gammtdraw gammtcdraw gammwgdraw sigadraw sigbdraw...
        sigmdraw sigidraw siglabdraw sigwdraw sigpdraw siggdraw sigtdraw sigtcdraw...
        sigzdraw sigwgdraw rhowgdraw kappawpdraw mundraw thetaepdraw thetaewdraw thetaLdraw];

finalaccept = acceptcount/simlen;

toc
    
    
    