% get_posterior_irf.m

load mhresults;
load mhresults_final
load modes;
burnin = 500000;            % How many draws to burn.
thin = 100;                 % How many draws to thin.  (a value of one implies no draws are thinned)
draw2 = draw(burnin+1:thin:end,:);
postdraw2 = postdraw(burnin+1:thin:end,:);
ndraw = size(draw2,1);
nparam = size(draw2,2);

%% Initializations
nvar = 68;          % number of variables
nexog = 12;         % number of exogenous shocks
nforerrors = 7;     % number of forecast errors
steps = 161;         % time horizon for impulse responses
conflevel = .90;    % 90% error band
%% Pre-specify matrices
posterior_irf = zeros(ndraw,nvar,nexog,steps);
post_mean = zeros(nvar,nexog,steps);
post_median = zeros(nvar,nexog,steps);
post_hpd = zeros(2,nvar,nexog,steps);
declare_variable;

standard_deviation_shock = 0;
%% Calculate impulse responses


parfor iz = 1:ndraw
    param = draw2(iz,:);
    [g0,g1,CC,Psi,Pie,H,C,E,ssvalues,ssnames] = model_wgcut(param',bnames,obs,glevelI);
    [G,C0,M,fmat,fwt,ywt,gev,eu] = gensys(g0,g1,CC,Psi,Pie);
    M=M(:,nvar-nexog+1:end);
    imf = zeros(nvar,nexog,steps);	

    if standard_deviation_shock == 1
        imf(:,:,1) = M*E;
    else
        imf(:,:,1) = M;
    end
    for t = 2:steps
        imf(:,:,t) = G*imf(:,:,t-1);
    end
    %eliminate very small responses
    for i = 1:nvar
        for j = 1:nexog
            for k = 1:steps
                if abs(imf(i,j,k))<10^-10
                    imf(i,j,k) = 0;
                end
            end
        end
    end   
    posterior_irf(iz,:,:,:) = imf;
end

for ii = 1:nvar
    for jj = 1:nexog
        for kk = 1:steps
            irfpoint = posterior_irf(:,ii,jj,kk);
            irfpoint = sort(irfpoint);
            post_mean(ii,jj,kk) = mean(irfpoint);
            post_median(ii,jj,kk) = median(irfpoint);

            number_of_draws = length(irfpoint);
            hpd_draws = round((1 - conflevel)*number_of_draws);
            qq = zeros(hpd_draws,1);
            hh = number_of_draws - hpd_draws;
            for vv = 1:hpd_draws
                qq(vv) = irfpoint(hh) - irfpoint(vv);
                hh = hh + 1;
            end
            [kmin,index] = min(qq);
            hpd_interval = [irfpoint(index) irfpoint(index) + kmin];
            post_hpd(:,ii,jj,kk) = hpd_interval(1,:);
        end
    end
end

if standard_deviation_shock == 1
    save postIRF_std_wgcut post_hpd post_median post_mean 
else
    save postIRF_wgcut post_hpd post_median post_mean 
end

