
load mhresults;
load mhresults_final
load modes;
burnin = 500000;            % How many draws to burn.
thin = 100;                 % How many draws to thin.  (a value of one implies no draws are thinned)
draw2 = draw(burnin+1:thin:end,:);
postdraw2 = postdraw(burnin+1:thin:end,:);
ndraw = size(draw2,1);
nparam = size(draw2,2);

%% Initializations
nvar = 68;          % number of variables
nexog = 12;         % number of exogenous shocks
nforerrors = 7;     % number of forecast errors
steps = 161;         % time horizon for impulse responses
declare_variable;
shk =[Nug Nuwg Nut Nuz]; % which shocks to compute multipliers for
conflevel = .90;
%% Pre-specify matrices
mY_posterior = zeros(ndraw,steps,size(shk,2)); 
mYP_posterior = zeros(ndraw,steps,size(shk,2));
mYG_posterior = zeros(ndraw,steps,size(shk,2)); 
mC_posterior = zeros(ndraw,steps,size(shk,2));
mI_posterior = zeros(ndraw,steps,size(shk,2));
mg_posterior = zeros(ndraw,steps,size(shk,2)); 
mt_posterior = zeros(ndraw,steps,size(shk,2));
mz_posterior = zeros(ndraw,steps,size(shk,2));
mfs_posterior = zeros(ndraw,steps,size(shk,2));
%% Calculate multipliers


parfor iz = 1:ndraw
    param = draw2(iz,:);
    [g0,g1,CC,Psi,Pie,H,C,E,ssvalues,ssnames] = model_wgcut(param',bnames,obs,glevelI);
    [G,C0,M,fmat,fwt,ywt,gev,eu] = gensys(g0,g1,CC,Psi,Pie);
    omega = eye(nexog);
    omegafill = zeros(nvar,nvar);
    omegafill(nvar-nexog+1:nvar,nvar-nexog+1:nvar)=omega;

    Bh = G';
    swish = M*omegafill;  % covariance matrix
    lags = 1;
    nn = [nvar lags steps];
    imf = impulse0(Bh,swish,nn);
    
    rrealtemp = zeros(steps,size(shk,2));  
    Ytemp = zeros(steps,size(shk,2));
    YPtemp = zeros(steps,size(shk,2));
    YGtemp = zeros(steps,size(shk,2));
    Ctemp = zeros(steps,size(shk,2));
    Itemp =  zeros(steps,size(shk,2));
    gtemp = zeros(steps,size(shk,2)); 
    ttemp = zeros(steps,size(shk,2));
    ztemp = zeros(steps,size(shk,2));
    fstemp = zeros(steps,size(shk,2));
    
    mY = zeros(steps,size(shk,2)); 
    mYP = zeros(steps,size(shk,2));
    mYG = zeros(steps,size(shk,2)); 
    mC = zeros(steps,size(shk,2));
    mI = zeros(steps,size(shk,2));
    mg = zeros(steps,size(shk,2)); 
    mt = zeros(steps,size(shk,2));
    mz = zeros(steps,size(shk,2));
    mfs = zeros(steps,size(shk,2));
    
    discY = zeros(steps,1);
    discYP = zeros(steps,1);
    discYG = zeros(steps,1);
    discC = zeros(steps,1);
    discI = zeros(steps,1);
    discg = zeros(steps,1);
    disct = zeros(steps,1);
    discz = zeros(steps,1);
    discfs = zeros(steps,1);
  
    disc = zeros(steps,size(shk,2));
    disc(1,:) = ones(1,size(shk,2));

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %Compute multipliers
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    rk = ssvalues(1); R = ssvalues(2); psi1 = ssvalues(3); wp = ssvalues(4); wg = ssvalues(5); ygy = ssvalues(6);
    ypy = ssvalues(7); lpyp = ssvalues(8); klp = ssvalues(9); cyp = ssvalues(10); str = ssvalues(11); 
    y = ssvalues(12); l = ssvalues(13); Omeg = ssvalues(14); w = ssvalues(15); lp = ssvalues(16); lg = ssvalues(17);
    yg = ssvalues(18); yp = ssvalues(19); gd = ssvalues(20); GT = ssvalues(21); k = ssvalues(22); inv_ss = ssvalues(23);
    z = ssvalues(24); b = ssvalues(25); c = ssvalues(26); lambr = ssvalues(27); t = ssvalues(28); rreal = ssvalues(29);
    mun = ssvalues(30); varphi = ssvalues(31); sb = ssvalues(32);fs = ssvalues(33); fd = ssvalues(34);


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
    for k = 1:size(shk,2) 
        rrealtemp(:,k) = rreal*(1+imf(:,nvar*(shk(k)-1)+Nr));   
        Ytemp(:,k) = y*(imf(:,nvar*(shk(k)-1)+Ny));
        YPtemp(:,k) = yp*(imf(:,nvar*(shk(k)-1)+Nyp));
        YGtemp(:,k) = yg*(imf(:,nvar*(shk(k)-1)+Nyg));
        Ctemp(:,k) = c*(imf(:,nvar*(shk(k)-1)+Nc));
        Itemp(:,k) =  inv_ss*(imf(:,nvar*(shk(k)-1)+Ni));
        gtemp(:,k) = gd*(imf(:,nvar*(shk(k)-1)+Ngd)); 
        ttemp(:,k) = t*(imf(:,nvar*(shk(k)-1)+Nt));
        ztemp(:,k) = z*(imf(:,nvar*(shk(k)-1)+Nz));
        fstemp(:,k) = -fs*(imf(:,nvar*(shk(k)-1)+Nfs));
    end

    %%%%% 
    for i = 2:steps
        for k = 1:size(shk,2)
            disc(i,k) = disc(i-1,k)*rrealtemp(i,k);
        end
    end

    for i=1:steps
        for k = 1:size(shk,2)   
            discY(i,k) = Ytemp(i,k)/disc(i,k);
            discYP(i,k) = YPtemp(i,k)/disc(i,k);
            discYG(i,k) = YGtemp(i,k)/disc(i,k);
            discC(i,k) = Ctemp(i,k)/disc(i,k);
            discI(i,k) = Itemp(i,k)/disc(i,k); 
            discg(i,k)= gtemp(i,k)/disc(i,k);  
            disct(i,k)= ttemp(i,k)/disc(i,k);  
            discz(i,k)= ztemp(i,k)/disc(i,k);  
            discfs(i,k)= fstemp(i,k)/disc(i,k);
        end
    end

    for i = 1:steps
            mY(i,1) = sum(discY(1:i,1))/sum(discg(1:i,1));
            mYP(i,1) = sum(discYP(1:i,1))/sum(discg(1:i,1));
            mYG(i,1) = sum(discYG(1:i,1))/sum(discg(1:i,1));
            mC(i,1) = sum(discC(1:i,1))/sum(discg(1:i,1));
            mI(i,1) = sum(discI(1:i,1))/sum(discg(1:i,1));
            mg(i,1) = sum(discg(1:i,1))/sum(discg(1:i,1));  
            mt(i,1) = sum(disct(1:i,1))/sum(discg(1:i,1)); 
            mz(i,1) = sum(discz(1:i,1))/sum(discg(1:i,1)); 
            mfs(i,1) = sum(discfs(1:i,1))/sum(discg(1:i,1));
            
            mY(i,2) = sum(discY(1:i,2))/sum(discYG(1:i,2));
            mYP(i,2) = sum(discYP(1:i,2))/sum(discYG(1:i,2));
            mYG(i,2) = sum(discYG(1:i,2))/sum(discYG(1:i,2));
            mC(i,2) = sum(discC(1:i,2))/sum(discYG(1:i,2));
            mI(i,2) = sum(discI(1:i,2))/sum(discYG(1:i,2));
            mg(i,2) = sum(discg(1:i,2))/sum(discYG(1:i,2));  
            mt(i,2) = sum(disct(1:i,2))/sum(discYG(1:i,2)); 
            mz(i,2) = sum(discz(1:i,2))/sum(discYG(1:i,2)); 
            mfs(i,2) = sum(discfs(1:i,2))/sum(discYG(1:i,2));
            
            mY(i,3) = sum(discY(1:i,3))/sum(disct(1:i,3));
            mYP(i,3) = sum(discYP(1:i,3))/sum(disct(1:i,3));
            mYG(i,3) = sum(discYG(1:i,3))/sum(disct(1:i,3));
            mC(i,3) = sum(discC(1:i,3))/sum(disct(1:i,3));
            mI(i,3) = sum(discI(1:i,3))/sum(disct(1:i,3));
            mg(i,3) = sum(discg(1:i,3))/sum(disct(1:i,3));  
            mt(i,3) = sum(disct(1:i,3))/sum(disct(1:i,3)); 
            mz(i,3) = sum(discz(1:i,3))/sum(disct(1:i,3));
            mfs(i,3) = sum(discfs(1:i,3))/sum(disct(1:i,3)); 
            
            mY(i,4) = sum(discY(1:i,4))/sum(discz(1:i,4));
            mYP(i,4) = sum(discYP(1:i,4))/sum(discz(1:i,4));
            mYG(i,4) = sum(discYG(1:i,4))/sum(discz(1:i,4));
            mC(i,4) = sum(discC(1:i,4))/sum(discz(1:i,4));
            mI(i,4) = sum(discI(1:i,4))/sum(discz(1:i,4));
            mg(i,4) = sum(discg(1:i,4))/sum(discz(1:i,4));  
            mt(i,4) = sum(disct(1:i,4))/sum(discz(1:i,4)); 
            mz(i,4) = sum(discz(1:i,4))/sum(discz(1:i,4)); 
            mfs(i,4) = sum(discfs(1:i,4))/sum(discz(1:i,4)); 

    end
    mY_posterior(iz,:,:) = mY; 
    mYP_posterior(iz,:,:) = mYP;
    mYG_posterior(iz,:,:) = mYG; 
    mC_posterior(iz,:,:) = mC;
    mI_posterior(iz,:,:) = mI;
    mg_posterior(iz,:,:) = mg; 
    mt_posterior(iz,:,:) = mt;
    mz_posterior(iz,:,:) = mz;
    mfs_posterior(iz,:,:) = mfs;
end

%% Calculate bands
for ii = 1:steps
    for jj = 1:size(shk,2)
        mYpoint = mY_posterior(:,ii,jj);
        mYpoint = sort(mYpoint);
        mY_mean(:,ii,jj) = mean(mYpoint);
        mY_median(:,ii,jj) = median(mYpoint);
        mYPpoint = mYP_posterior(:,ii,jj);
        mYPpoint = sort(mYPpoint);
        mYP_mean(:,ii,jj) = mean(mYPpoint);
        mYP_median(:,ii,jj) = median(mYPpoint);
        mYGpoint = mYG_posterior(:,ii,jj);
        mYGpoint = sort(mYGpoint);
        mYG_mean(:,ii,jj) = mean(mYGpoint);
        mYG_median(:,ii,jj) = median(mYGpoint);
        mCpoint = mC_posterior(:,ii,jj);
        mCpoint = sort(mCpoint);
        mC_mean(:,ii,jj) = mean(mCpoint);
        mC_median(:,ii,jj) = median(mCpoint);
        mIpoint = mI_posterior(:,ii,jj);
        mIpoint = sort(mIpoint);
        mI_mean(:,ii,jj) = mean(mIpoint);
        mI_median(:,ii,jj) = median(mIpoint);
        mfspoint = mfs_posterior(:,ii,jj);
        mfspoint = sort(mfspoint);
        mfs_mean(:,ii,jj) = mean(mfspoint);
        mfs_median(:,ii,jj) = median(mfspoint);
        
        number_of_draws = length(mYpoint);
        hpd_draws = round((1 - conflevel)*number_of_draws);
        qq = zeros(hpd_draws,1);
        hh = number_of_draws - hpd_draws;

        for vv = 1:hpd_draws
            qq_mY(vv) = mYpoint(hh) - mYpoint(vv);
            qq_mYP(vv) = mYPpoint(hh) - mYPpoint(vv);
            qq_mYG(vv) = mYGpoint(hh) - mYGpoint(vv);
            qq_mC(vv) = mCpoint(hh) - mCpoint(vv);
            qq_mI(vv) = mIpoint(hh) - mIpoint(vv);
            qq_mfs(vv) = mfspoint(hh) - mfspoint(vv);
            hh = hh + 1;
        end
        [kmin,index] = min(qq_mY);
        hpd_interval = [mYpoint(index) mYpoint(index) + kmin];
        post_hpd_mY(:,ii,jj) = hpd_interval(1,:);
        [kmin,index] = min(qq_mYP);
        hpd_interval = [mYPpoint(index) mYPpoint(index) + kmin];
        post_hpd_mYP(:,ii,jj) = hpd_interval(1,:);
        [kmin,index] = min(qq_mYG);
        hpd_interval = [mYGpoint(index) mYGpoint(index) + kmin];
        post_hpd_mYG(:,ii,jj) = hpd_interval(1,:);
        [kmin,index] = min(qq_mC);
        hpd_interval = [mCpoint(index) mCpoint(index) + kmin];
        post_hpd_mC(:,ii,jj) = hpd_interval(1,:);
        [kmin,index] = min(qq_mI);
        hpd_interval = [mIpoint(index) mIpoint(index) + kmin];
        post_hpd_mI(:,ii,jj) = hpd_interval(1,:);
        [kmin,index] = min(qq_mfs);
        hpd_interval = [mfspoint(index) mfspoint(index) + kmin];
        post_hpd_mfs(:,ii,jj) = hpd_interval(1,:);
    end
end

save post_multipliers_wgcut post_hpd_mY post_hpd_mYP post_hpd_mYG post_hpd_mC post_hpd_mI post_hpd_mfs...
    mY_mean mY_median mYP_mean mYP_median mYG_mean mYG_median mC_mean ...
    mC_median mI_mean mI_median mfs_mean mfs_median

save post_multipliers_total_wgcut mY_posterior mYP_posterior mYG_posterior mC_posterior mI_posterior mfs_posterior...
    mg_posterior mt_posterior mz_posterior

