function neg_log_post = logpostmodel(param,bnames)

nvar = 68;          % number of variables
nexog = 12;         % number of exogenous shocks
nforerrors = 7;     % number of forecast errors

logprior = priors(param);
if logprior == -Inf
    neg_log_post = Inf;
else

load logpostinfo

[g0,g1,CC,Psi,Pie,H,C,E,ssvalues,ssnames,bad_steady_state] = model(param',bnames,obs);

if bad_steady_state == 1
    neg_log_post = Inf;
else
    [G,C0,M,fmat,fwt,ywt,gev,eu] = gensys(g0,g1,CC,Psi,Pie);
    M=M(:,nvar-nexog+1:end);
    %eu
    if eu(1,1) ~= 1 || eu(2,1) ~= 1 
        % If no determinate equilibrium, bad parameter values.
        neg_log_post = Inf;
    else % Set up matrices for Kalman filter
        Q = M*E*E'*M';
        initP = lyapunov_symm(G,Q,1); % lyapunov_symm is a dynare code that is a
        % fast way to solve a Lyapunov equation symmetrically
        
        initX = zeros(nvar,1);
        nobs = size(obsdata,1);
        T = size(obsdata,2);
        
        loglike = kalman(obsdata,G,Q,initP,initX,T,nobs,obs,C);
        
        neg_log_post = -(loglike + logprior);
    end

end

end
