% =========================================================================
% Code for "Fiscal Consolidation and Public Wages"
% by Juin-Jen Chang, Hsieh-Yu Lin, Nora Traum, and Shu-Chun S. Yang
%
% mainscript.m
% This code estimates the model. 
% =========================================================================
clear 
clc
rand('state',759);
randn('state',489);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   User Initializations   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
modesearchI = 1;               % 1 = perform posterior mode search; 0 otherwise
get_posteriorI = 1;            % 1 = construct posterior; 0 otherwise 
 

d1 = 1984.1;                   % first quarter in estimation
d2 = 2016.4;                   % last quarter in estimation


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Other Initializations   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nvar = 68;          % number of variables
nexog = 12;         % number of exogenous shocks
nforerrors = 7;     % number of forecast errors
nparam = 43;        % number of estimated parameters
nobs = nexog;       % number of observables

% Define variables from DSGE model
declare_variable;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Pick Observables      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
obs = zeros(nobs,1);
obs(1,1) = Ncobs;
obs(2,1) = Niobs;
obs(3,1) = Nwpobs;  
obs(4,1) = Nwgobs;
obs(5,1) = Ngdobs;
obs(6,1) = Nbobs;
obs(7,1) = NTobs;
obs(8,1) = NTcobs;
obs(9,1) = NLpobs;
obs(10,1) = NPiobs;
obs(11,1) = NRobs;
obs(12,1) = Nunempobs;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%       Get Data           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[obsdata] = get_data(d1,d2);
save logpostinfo.mat obs obsdata


if modesearchI == 1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Find Posterior Mode     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve using Chris Sims' optimization program, csminwel
Hinit = 1e-4*eye(nparam);
num = 5000; % number of prior searches
mnum = 25;% 

tic
[neg_log_post,param_total,bnames] = searchprior(num,nparam);
[param_mode, xhat,Hhat, fhat, retcode] = findmode(neg_log_post,param_total,mnum,Hinit,bnames);
toc

save modes.mat xhat Hhat fhat param_mode retcode obs obsdata bnames
[opt] = optmode(fhat,Hhat,retcode);
end

if get_posteriorI == 1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Load Posterior Mode     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load modes.mat
[opt] = optmode(fhat,Hhat,retcode);

%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Run MH Algorithm     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%
simlen = 2000000;
[draw, finalaccept,postdraw, priorcount, eucount] = MHalgorithm(obsdata,simlen,xhat(:,opt),Hhat(:,:,opt),nvar,nforerrors,nexog,obs,bnames);
save mhresults_final.mat draw finalaccept postdraw xhat Hhat fhat obsdata opt priorcount eucount bnames
end




