function [p] = priors(param)
% function [p] = priors(param)


param = param';
p = 0;

%% prior on gamm100: steady state growth rate of technology, multiplied by 100
param_index = 1;
p = log(normpdf(param(param_index,1),0.5,0.03)) + p;    

%% prior on kappa: inverse Frisch elasticity; MEAN = 2; STD = 0.75 
param_index = param_index + 1;
p = log_gamma_pdf(param(param_index,1),2,0.75) + p;              

%% prior on omegaw: prob. of unions not resetting price, MEAN = 0.66; STD = 0.1
param_index = param_index + 1;
p = log_beta_pdf(param(param_index,1),0.5,0.1) + p;   

%% prior on omegap: prob. of intermediate firms not resetting price, MEAN = 0.66; STD = 0.1 
param_index = param_index + 1;
p = log_beta_pdf(param(param_index,1),0.5,0.1) + p; 


%% prior on phipi: Taylor rule inflation coefficient, MEAN = 1.5; STD = 0.25 
param_index = param_index + 1;
p = log(normpdf(param(param_index,1),1.5,0.25)) + p;              

%% prior on phiY: Taylor rule output coefficient, MEAN = 0.125; STD = 0.05 
param_index = param_index + 1;
p = log(normpdf(param(param_index,1),0.125,0.05)) + p; 

%% prior on habit formation, MEAN = 0.5; STD = 0.2
param_index = param_index + 1;
p = log_beta_pdf(param(param_index,1),0.5,0.2) + p; 

%% prior on psi: K utilization cost parameter, MEAN = 0.5, STD = 0.2 
param_index = param_index + 1;
p = log_beta_pdf(param(param_index,1),0.5,0.2) + p;       

%% prior on s: s" in the investment adjustment cost function, MEAN = 5, STD = 1.5
param_index = param_index + 1;
p = log(normpdf(param(param_index,1),5,1.5)) + p;        

%% prior on chiw: percentage of unions' wage adjustment when they cannot optimize, MEAN = 0.5, STD = 0.15
param_index = param_index + 1;
p = log_beta_pdf(param(param_index,1),0.5,0.15) + p;       


%% prior on tech serial correlation, MEAN: 0.5, STD = 0.2 
param_index = param_index + 1;
p = log_beta_pdf(param(param_index,1),0.5,0.15) + p; 

%% prior on preference serial correlation, MEAN: 0.7, STD = 0.2 
param_index = param_index + 1;
p = log_beta_pdf(param(param_index,1),0.5,0.15) + p; 

%% prior on monetary policy serial correlation, MEAN: 0.7, STD = 0.2 
param_index = param_index + 1;
p = log_beta_pdf(param(param_index,1),0.5,0.15) + p; 

%% prior on investment serial correlation, MEAN: 0.7, STD = 0.2 
param_index = param_index + 1; 
p = log_beta_pdf(param(param_index,1),0.5,0.15) + p; 

%% prior on labor preference serial correlation, MEAN: 0.5, STD = 0.15 
param_index = param_index + 1;
p = log_beta_pdf(param(param_index,1),0.5,0.15) + p; 

%% prior on wage shock serial correlation, MEAN: 0.7, STD = 0.2 
param_index = param_index + 1;
p = log_beta_pdf(param(param_index,1),0.5,0.15) + p; 

%% prior on price shock serial correlation, MEAN: 0.7, STD = 0.2 
param_index = param_index + 1;
p = log_beta_pdf(param(param_index,1),0.5,0.15) + p; 

%% prior on gov spending serial correlation, MEAN: 0.7, STD = 0.2 
param_index = param_index + 1; 
p = log_beta_pdf(param(param_index,1),0.5,0.15) + p; 

%% prior on tax serial correlation, MEAN: 0.7, STD = 0.2 
param_index = param_index + 1;
p = log_beta_pdf(param(param_index,1),0.5,0.15) + p; 

%% prior on consumption tax serial correlation, MEAN: 0.7, STD = 0.2 
param_index = param_index + 1;
p = log_beta_pdf(param(param_index,1),0.5,0.15) + p; 

%% prior on transfer serial correlation, MEAN: 0.7, STD = 0.2 
param_index = param_index + 1;
p = log_beta_pdf(param(param_index,1),0.5,0.15) + p;  

%% prior on gammg: response of government demand to debt, MEAN: 0.05, STD = 0.02
param_index = param_index + 1;
p = log(normpdf(param(param_index,1),0.2,0.03)) + p; 

%% prior on gammt: response of taxes to debt, MEAN: 0.05, STD = 0.02
param_index = param_index + 1;
p = log(normpdf(param(param_index,1),0.2,0.03)) + p; 

%% prior on gammtc: response of consumption taxes to debt, MEAN: 0.05, STD = 0.02
param_index = param_index + 1;  
p = log(normpdf(param(param_index,1),0.2,0.03)) + p; 

%% prior on gammwg: response of taxes to debt, MEAN: 0.1, STD = 0.05
param_index = param_index + 1;
p = log(normpdf(param(param_index,1),0.2,0.03)) + p; 


%% prior on tech precision
param_index = param_index + 1;
[s,nu] = inverse_gamma_specification(0.1,1,1);
p = lpdfig1(param(param_index,1),s,nu) + p;  

%% prior on cons. precision
param_index = param_index + 1;
[s,nu] = inverse_gamma_specification(0.1,1,1);
p = lpdfig1(param(param_index,1),s,nu) + p;  

%% prior on money precision
param_index = param_index + 1;
[s,nu] = inverse_gamma_specification(0.1,1,1);
p = lpdfig1(param(param_index,1),s,nu) + p;    

%% prior on investment precision
param_index = param_index + 1;
[s,nu] = inverse_gamma_specification(0.1,1,1);
p = lpdfig1(param(param_index,1),s,nu) + p;  

%% prior on labor preference precision
param_index = param_index + 1;
[s,nu] = inverse_gamma_specification(0.1,1,1);
p = lpdfig1(param(param_index,1),s,nu) + p;  

%% prior on wage markup precision
param_index = param_index + 1;
[s,nu] = inverse_gamma_specification(0.1,1,1);
p = lpdfig1(param(param_index,1),s,nu) + p;  

%% prior on price markup precision
param_index = param_index + 1;
[s,nu] = inverse_gamma_specification(0.1,1,1);
p = lpdfig1(param(param_index,1),s,nu) + p;  

%% prior on gov. demand precision
param_index = param_index + 1;
[s,nu] = inverse_gamma_specification(1,1,1);
p = lpdfig1(param(param_index,1),s,nu) + p;     

%% prior on tax precision
param_index = param_index + 1;
[s,nu] = inverse_gamma_specification(1,1,1);
p = lpdfig1(param(param_index,1),s,nu) + p;

%% prior on consumption tax precision
param_index = param_index + 1;
[s,nu] = inverse_gamma_specification(1,1,1);
p = lpdfig1(param(param_index,1),s,nu) + p;

%% prior on transfer precision
param_index = param_index + 1;
[s,nu] = inverse_gamma_specification(1,1,1);
p = lpdfig1(param(param_index,1),s,nu) + p;

%% prior on public wage precision
param_index = param_index + 1;
[s,nu] = inverse_gamma_specification(1,1,1);
p = lpdfig1(param(param_index,1),s,nu) + p; 

%% prior on rhowg, MEAN: 0.7, STD = 0.2 
param_index = param_index + 1;
p = log_beta_pdf(param(param_index,1),0.5,0.15) + p; 

%% prior on kappawp, MEAN: 0.5, STD = 0.2 
param_index = param_index + 1;
p = log(normpdf(param(param_index,1),0.5,0.2)) + p; 

%% prior on mun: elasticity of substitution between the labor used in private and public sectors; MEAN = 3; STD = 1 
param_index = param_index + 1;
p = log_gamma_pdf(param(param_index,1),3,1) + p;  

%% prior on thetaep
param_index = param_index + 1;
p = log_beta_pdf(param(param_index,1),0.5,0.15) + p; 

%% prior on thetaew
param_index = param_index + 1;
p = log_beta_pdf(param(param_index,1),0.5,0.15) + p; 

%% prior on thetaL
param_index = param_index + 1;
p = log_beta_pdf(param(param_index,1),0.5,0.2) + p; 
