function [neg_log_post,param_total,bnames] = searchprior(num,nparam)

neg_log_post = zeros(num,1);
param_total = zeros(num,nparam);

alpha_beta      = 'mu*(mu*(1-mu)/sig^2-1)';
beta_beta       = '(1-mu)*(mu*(1-mu)/sig^2-1)';
alpha_gamma     = 'mu/((sig^2)/mu)';
beta_gamma      = '(sig^2)/mu';
alpha_invgamma  = '2 + mu/(sig^2)';  
beta_invgamma   = 'mu*(1 + mu/(sig^2))';

disp('Taking Initial Draws From Priors')
for q = 1:num
    
    if q/100 == round(q/100)
        disp([q])
    end
    param = zeros(nparam,1);
    param_index = 1;
    
    %% steady state log growth rate of technology, multiplied by 100
    bnames = 'gamm100';
    mu = 0.5; sig = 0.03; 
    param(param_index,1) = normrnd(mu,sig);
    clear mu sig                 
    param_index = param_index + 1;
    
    %% curvature over leisure
    bnames = char(bnames,'kappa');
    mu = 2; sig = 0.75; 
    param(param_index,1) = gamrnd(eval(alpha_gamma),eval(beta_gamma)); clear mu sig                 
    param_index = param_index + 1;  
    
    %% prob that union cannot reset optimal wage
    bnames = char(bnames,'omegaw');
    mu = 0.5; sig = 0.1;
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                   
    param_index = param_index + 1;     
    
    %% prob that intermediat firms cannot reset optimal price
    bnames = char(bnames,'omegap');
    mu = 0.5; sig = 0.1;
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                   
    param_index = param_index + 1;    
    
    %% Taylor rule inflation coefficient
    bnames = char(bnames,'phipi');
    mu = 1.5; sig = 0.25; 
    param(param_index,1) = normrnd(mu,sig); clear mu sig                   
    param_index = param_index + 1;     
    
    %% Taylor rule output coefficient
    bnames = char(bnames,'phiy');
    mu = 0.125; sig = 0.05; 
    param(param_index,1) = normrnd(mu,sig); clear mu sig                   
    param_index = param_index + 1;      
    
    %% habit formation parameter
    bnames = char(bnames,'theta');
    mu = 0.5; sig = 0.2; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;      
    
    %% K utilization cost parameter
    bnames = char(bnames,'gpsi');
    mu = 0.5; sig = 0.2; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;    
    
    %% s" in the investment adjustment cost function
    bnames = char(bnames,'s');
    mu = 5; sig = 1.5; 
    param(param_index,1) = normrnd(mu,sig); clear mu sig                   
    param_index = param_index + 1; 
    
    %% the percent of unions' wage adjustment when they cannot optimize 
    bnames = char(bnames,'chiw');
    mu = 0.5; sig = 0.15; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;     
   
    %% serial correlation on a 
    bnames = char(bnames,'rhoa');
    mu = 0.5; sig = 0.15; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1; 
    
    %% serial correlation on cons. preference (ub)
    bnames = char(bnames,'rhob');
    mu = 0.5; sig = 0.15; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;  
    
    %% serial correlation on interest rate in Taylor rule
    bnames = char(bnames,'rhor');
    mu = 0.5; sig = 0.15; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;  
  
    %% serial correlation on ui
    bnames = char(bnames,'rhoi');
    mu = 0.5; sig = 0.15; 
     param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;  
    
        %% serial correlation on ulab
    bnames = char(bnames,'rholab');
    mu = 0.5; sig = 0.15; 
     param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;  
    
    %% serial correlation on uw
    bnames = char(bnames,'rhow');
    mu = 0.5; sig = 0.15; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1; 
    
    %% serial correlation on up
    bnames = char(bnames,'rhop');
    mu = 0.5; sig = 0.15;  
     param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;    
    
    %% serial correlation on g
    bnames = char(bnames,'rhog');
    mu = 0.5; sig = 0.15; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;  
  
    %% serial correlation on tau
    bnames = char(bnames,'rhot');
    mu = 0.5; sig = 0.15;   
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;  
    
    %% serial correlation on tauc
    bnames = char(bnames,'rhotc');
    mu = 0.5; sig = 0.15;  
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;  
    
    %% serial correlation on transfers
    bnames = char(bnames,'rhoz');
    mu = 0.5; sig = 0.15; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;  
    
    %% response of G to debt
    bnames = char(bnames,'gammg');
    mu = 0.2; sig = 0.03; 
    param(param_index,1) = normrnd(mu,sig); clear mu sig                    
    param_index = param_index + 1;  
    
    %% response of income tax revenue to debt
    bnames = char(bnames,'gammt');
    mu = 0.2; sig = 0.03; 
    param(param_index,1) = normrnd(mu,sig); clear mu sig                    
    param_index = param_index + 1;  
    
    %% response of consumption tax revenue to debt
    bnames = char(bnames,'gammtc');
    mu = 0.2; sig = 0.03; 
    param(param_index,1) = normrnd(mu,sig); clear mu sig                    
    param_index = param_index + 1;  
    
    %% response of W^G to debt
    bnames = char(bnames,'gammwg');
    mu = 0.2; sig = 0.03; 
    param(param_index,1) = normrnd(mu,sig); clear mu sig                    
    param_index = param_index + 1;  
    
    %% standard deviation technology growth rate
    bnames = char(bnames,'siga');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation preference
    bnames = char(bnames,'sigb');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation monetary policy
    bnames = char(bnames,'sigm');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation investment
    bnames = char(bnames,'sigi');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
        %% standard deviation labor preference
    bnames = char(bnames,'siglab');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation wage markup
    bnames  = char(bnames,'sigw');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation price markup
    bnames = char(bnames,'sigp');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation gov. spending
    bnames = char(bnames,'sigg');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation taxes
    bnames = char(bnames,'sigt');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation consumption taxes
    bnames = char(bnames,'sigtc');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation transfers
    bnames = char(bnames,'sigz');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation public wages
    bnames = char(bnames,'sigwg');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% persistence public wages
    bnames = char(bnames,'rhowg');
    mu = 0.5; sig = 0.15; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig  
    param_index = param_index + 1; 
    
    %% response of public wage to private wage
    bnames = char(bnames,'kappawp');
    mu = 0.5; sig = 0.2; 
    param(param_index,1) = normrnd(mu,sig); clear mu sig                  
    param_index = param_index + 1; 
    
    %% elasticity of substitution between the labor used in private and public sectors
    bnames = char(bnames,'mun');
    mu = 3; sig = 1; 
    param(param_index,1) = gamrnd(eval(alpha_gamma),eval(beta_gamma)); clear mu sig                  
    param_index = param_index + 1; 
    
    %% serial correlation on thetaep
   bnames = char(bnames,'thetaep');
   mu = 0.5; sig = 0.15;
   param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
   param_index = param_index + 1;  
    
    %% serial correlation on thetaew
   bnames = char(bnames,'thetaew');
   mu = 0.5; sig = 0.15;
   param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig  
    param_index = param_index + 1;  
    
       %% serial correlation on thetaL
   bnames = char(bnames,'thetaL');
   mu = 0.5; sig = 0.2;
   param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig 
    
    %% Evaluate likelihood
    neg_log_post(q,:) = logpostmodel(param',bnames);  % negative log posterior.
    param_total(q,:) = param';
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Repeat: keep lookping until good parameter draw
    while real(neg_log_post(q,:)) == inf                % initial value has to work
    param = zeros(nparam,1);
    param_index = 1;
    
   %% steady state log growth rate of technology, multiplied by 100
    bnames = 'gamm100';
    mu = 0.5; sig = 0.03; 
    param(param_index,1) = normrnd(mu,sig);
    clear mu sig                 
    param_index = param_index + 1;
    
    %% curvature over leisure
    bnames = char(bnames,'kappa');
    mu = 2; sig = 0.75; 
    param(param_index,1) = gamrnd(eval(alpha_gamma),eval(beta_gamma)); clear mu sig                 
    param_index = param_index + 1;  
    
    %% prob that union cannot reset optimal wage
    bnames = char(bnames,'omegaw');
    mu = 0.5; sig = 0.1;
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                   
    param_index = param_index + 1;     
    
    %% prob that intermediat firms cannot reset optimal price
    bnames = char(bnames,'omegap');
    mu = 0.5; sig = 0.1;
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                   
    param_index = param_index + 1;    
    
    %% Taylor rule inflation coefficient
    bnames = char(bnames,'phipi');
    mu = 1.5; sig = 0.25; 
    param(param_index,1) = normrnd(mu,sig); clear mu sig                   
    param_index = param_index + 1;     
    
    %% Taylor rule output coefficient
    bnames = char(bnames,'phiy');
    mu = 0.125; sig = 0.05; 
    param(param_index,1) = normrnd(mu,sig); clear mu sig                   
    param_index = param_index + 1;      
    
    %% habit formation parameter
    bnames = char(bnames,'theta');
    mu = 0.5; sig = 0.2; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;      
    
    %% K utilization cost parameter
    bnames = char(bnames,'gpsi');
    mu = 0.5; sig = 0.2; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;    
    
    %% s" in the investment adjustment cost function
    bnames = char(bnames,'s');
    mu = 5; sig = 1.5; 
    param(param_index,1) = normrnd(mu,sig); clear mu sig                   
    param_index = param_index + 1; 
    
    %% the percent of unions' wage adjustment when they cannot optimize 
    bnames = char(bnames,'chiw');
    mu = 0.5; sig = 0.15; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;     
   
    %% serial correlation on a 
    bnames = char(bnames,'rhoa');
    mu = 0.5; sig = 0.15; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1; 
    
    %% serial correlation on cons. preference (ub)
    bnames = char(bnames,'rhob');
    mu = 0.5; sig = 0.15; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;  
    
    %% serial correlation on interest rate in Taylor rule
    bnames = char(bnames,'rhor');
    mu = 0.5; sig = 0.15; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;  
  
    %% serial correlation on ui
    bnames = char(bnames,'rhoi');
    mu = 0.5; sig = 0.15; 
     param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;  
    
        %% serial correlation on ulab
    bnames = char(bnames,'rholab');
    mu = 0.5; sig = 0.15; 
     param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;
    
    %% serial correlation on uw
    bnames = char(bnames,'rhow');
    mu = 0.5; sig = 0.15; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1; 
    
    %% serial correlation on up
    bnames = char(bnames,'rhop');
    mu = 0.5; sig = 0.15;  
     param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;    
    
    %% serial correlation on g
    bnames = char(bnames,'rhog');
    mu = 0.5; sig = 0.15; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;  
  
    %% serial correlation on tau
    bnames = char(bnames,'rhot');
    mu = 0.5; sig = 0.15;   
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;  
    
    %% serial correlation on tauc
    bnames = char(bnames,'rhotc');
    mu = 0.5; sig = 0.15;  
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;  
    
    %% serial correlation on transfers
    bnames = char(bnames,'rhoz');
    mu = 0.5; sig = 0.15; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
    param_index = param_index + 1;  
    
    %% response of G to debt
    bnames = char(bnames,'gammg');
    mu = 0.2; sig = 0.03; 
    param(param_index,1) = normrnd(mu,sig); clear mu sig                    
    param_index = param_index + 1;  
    
    %% response of income tax revenue to debt
    bnames = char(bnames,'gammt');
    mu = 0.2; sig = 0.03; 
    param(param_index,1) = normrnd(mu,sig); clear mu sig                    
    param_index = param_index + 1;  
    
    %% response of consumption tax revenue to debt
    bnames = char(bnames,'gammtc');
    mu = 0.2; sig = 0.03; 
    param(param_index,1) = normrnd(mu,sig); clear mu sig                    
    param_index = param_index + 1;  
    
    %% response of W^G to debt
    bnames = char(bnames,'gammwg');
    mu = 0.2; sig = 0.03; 
    param(param_index,1) = normrnd(mu,sig); clear mu sig                    
    param_index = param_index + 1;  
    
    %% standard deviation technology growth rate
    bnames = char(bnames,'siga');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation preference
    bnames = char(bnames,'sigb');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation monetary policy
    bnames = char(bnames,'sigm');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation investment
    bnames = char(bnames,'sigi');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
            %% standard deviation labor preference
    bnames = char(bnames,'siglab');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation wage markup
    bnames  = char(bnames,'sigw');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation price markup
    bnames = char(bnames,'sigp');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation gov. spending
    bnames = char(bnames,'sigg');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation taxes
    bnames = char(bnames,'sigt');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation consumption taxes
    bnames = char(bnames,'sigtc');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation transfers
    bnames = char(bnames,'sigz');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% standard deviation public wages
    bnames = char(bnames,'sigwg');
    mu = .1; sig = 1;
    param(param_index,1) = 1/(gamrnd(eval(alpha_invgamma),eval(beta_invgamma))); clear mu sig
    param_index = param_index + 1;
    
    %% persistence public wages
    bnames = char(bnames,'rhowg');
    mu = 0.5; sig = 0.15; 
    param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig  
    param_index = param_index + 1; 
    
    %% response of public wage to private wage
    bnames = char(bnames,'kappawp');
    mu = 0.5; sig = 0.2; 
    param(param_index,1) = normrnd(mu,sig); clear mu sig                  
    param_index = param_index + 1; 
    
    %% elasticity of substitution between the labor used in private and public sectors
    bnames = char(bnames,'mun');
    mu = 3; sig = 1; 
    param(param_index,1) = gamrnd(eval(alpha_gamma),eval(beta_gamma)); clear mu sig                  
    param_index = param_index + 1; 
    
    %% serial correlation on thetaep
   bnames = char(bnames,'thetaep');
   mu = 0.5; sig = 0.15;
   param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig                    
   param_index = param_index + 1;  
    
    %% serial correlation on thetaew
   bnames = char(bnames,'thetaew');
   mu = 0.5; sig = 0.15;
   param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig  
    param_index = param_index + 1;  
    
       %% serial correlation on thetaL
   bnames = char(bnames,'thetaL');
   mu = 0.5; sig = 0.2;
   param(param_index,1) = betarnd(eval(alpha_beta),eval(beta_beta)); clear mu sig 
    
    %% Evaluate likelihood
    neg_log_post(q,:) = logpostmodel(param',bnames);  
    param_total(q,:) = param';
    end
end % loop over number of prior draws